/*
Copyright (C) 1997-2001 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
#include "../client/client.h"
#include "../client/qmenu.h"

//#define REF_SOFT	0
#define REF_OPENGL	0
//#define REF_3DFX	1
//#define REF_POWERVR	2
//#define REF_VERITE	3

extern cvar_t *vid_ref;
extern cvar_t *vid_fullscreen;
extern cvar_t *vid_gamma;
extern cvar_t *scr_viewsize;

static cvar_t *gl_mode;
static cvar_t *gl_driver;
static cvar_t *gl_picmip;
static cvar_t *gl_ext_palettedtexture;
static cvar_t *gl_finish;

//DANIEL
static cvar_t *gl_ext_pointparameters;
static cvar_t *gl_texturemode;
static cvar_t *gl_bitdepth;
static cvar_t *gl_dynamic;
static cvar_t *gl_shadows;

static cvar_t *sw_mode;
static cvar_t *sw_stipplealpha;

extern void M_ForceMenuOff( void );

/*
====================================================================

MENU INTERACTION

====================================================================
*/
#define SOFTWARE_MENU 0
#define OPENGL_MENU   1

static menuframework_s  s_software_menu;
static menuframework_s	s_opengl_menu;
static menuframework_s *s_current_menu;
static int				s_current_menu_index;

static menulist_s		s_mode_list[2];
static menulist_s		s_ref_list[2];
static menuslider_s		s_tq_slider;
static menuslider_s		s_screensize_slider[2];
static menuslider_s		s_brightness_slider[2];
static menulist_s  		s_fs_box[2];
static menulist_s  		s_stipple_box;
static menulist_s  		s_paletted_texture_box;

//DANIEL
//static menulist_s  		s_particle_type_box;
static menulist_s  		s_bitdepth_type_box;
static menulist_s		s_dynamiclight_type_box;
static menulist_s		s_texturemode_type_box;
static menulist_s		s_shadows_type_box;

static menulist_s  		s_finish_box;
static menuaction_s		s_cancel_action[2];
static menuaction_s		s_defaults_action[2];

static void DriverCallback( void *unused )
{
	s_ref_list[!s_current_menu_index].curvalue = s_ref_list[s_current_menu_index].curvalue;

	if ( s_ref_list[s_current_menu_index].curvalue == 0 )
	{
		s_current_menu = &s_software_menu;
		s_current_menu_index = 0;
	}
	else
	{
		s_current_menu = &s_opengl_menu;
		s_current_menu_index = 1;
	}

}

static void ScreenSizeCallback( void *s )
{
	menuslider_s *slider = ( menuslider_s * ) s;

	Cvar_SetValue( "viewsize", slider->curvalue * 10 );
}

static void BrightnessCallback( void *s )
{
	menuslider_s *slider = ( menuslider_s * ) s;

	if ( s_current_menu_index == SOFTWARE_MENU )
		s_brightness_slider[1].curvalue = s_brightness_slider[0].curvalue;
	else
		s_brightness_slider[0].curvalue = s_brightness_slider[1].curvalue;

	if ( stricmp( vid_ref->string, "soft" ) == 0 )
	{
		float gamma = ( 0.8 - ( slider->curvalue/10.0 - 0.5 ) ) + 0.5;

		Cvar_SetValue( "vid_gamma", gamma );
	}
}

static void ResetDefaults( void *unused )
{
	VID_MenuInit();
}

static void ApplyChanges( void *unused )
{
	float gamma;

	/*
	** make values consistent
	*/
	s_fs_box[!s_current_menu_index].curvalue = s_fs_box[s_current_menu_index].curvalue;
	s_brightness_slider[!s_current_menu_index].curvalue = s_brightness_slider[s_current_menu_index].curvalue;
	s_ref_list[!s_current_menu_index].curvalue = s_ref_list[s_current_menu_index].curvalue;

	/*
	** invert sense so greater = brighter, and scale to a range of 0.5 to 1.3
	*/
	gamma = ( 0.8 - ( s_brightness_slider[s_current_menu_index].curvalue/10.0 - 0.5 ) ) + 0.5;

	Cvar_SetValue( "vid_gamma", gamma );
	Cvar_SetValue( "sw_stipplealpha", s_stipple_box.curvalue );
	Cvar_SetValue( "gl_picmip", 3 - s_tq_slider.curvalue );
	Cvar_SetValue( "vid_fullscreen", s_fs_box[s_current_menu_index].curvalue );
	Cvar_SetValue( "gl_ext_palettedtexture", s_paletted_texture_box.curvalue );
	Cvar_SetValue( "gl_finish", s_finish_box.curvalue );
	Cvar_SetValue( "sw_mode", s_mode_list[SOFTWARE_MENU].curvalue );
	Cvar_SetValue( "gl_mode", s_mode_list[OPENGL_MENU].curvalue );

	// DANIEL
//	Cvar_SetValue( "gl_ext_pointparameters", s_particle_type_box.curvalue );
	Cvar_SetValue( "gl_bitdepth", s_bitdepth_type_box.curvalue*16 );
	Cvar_SetValue( "gl_dynamic", s_dynamiclight_type_box.curvalue );
	Cvar_SetValue( "gl_shadows", s_shadows_type_box.curvalue );

	if ( s_texturemode_type_box.curvalue == 0)
	{
	Cvar_Set( "gl_texturemode", "GL_LINEAR_MIPMAP_NEAREST" );
	}
	else if ( s_texturemode_type_box.curvalue == 1)
	{
	Cvar_Set( "gl_texturemode", "GL_LINEAR_MIPMAP_LINEAR" );
	}
	else
	{
	Cvar_Set( "gl_texturemode", "GL_LINEAR_MIPMAP_NEAREST" );
	}

	switch ( s_ref_list[s_current_menu_index].curvalue )
	{
/*	case REF_SOFT:
		//DANIEL SE TILL AT VI ALDRIG LADDAR SOFTWARE
		//		Cvar_Set( "vid_ref", "soft" );
		Cvar_Set( "vid_ref", "gl" );
		Cvar_Set( "gl_driver", "opengl32" );
		break;*/
	case REF_OPENGL:
		Cvar_Set( "vid_ref", "gl" );
		Cvar_Set( "gl_driver", "opengl32" );
		break;
/*	case REF_3DFX:
		Cvar_Set( "vid_ref", "gl" );
		Cvar_Set( "gl_driver", "3dfxgl" );
		break;
	case REF_POWERVR:
		Cvar_Set( "vid_ref", "gl" );
		Cvar_Set( "gl_driver", "pvrgl" );
		break;
	case REF_VERITE:
		Cvar_Set( "vid_ref", "gl" );
		Cvar_Set( "gl_driver", "veritegl" );
		break;*/
	}

	/*
	** update appropriate stuff if we're running OpenGL and gamma
	** has been modified
	*/
	if ( stricmp( vid_ref->string, "gl" ) == 0 )
	{
		if ( vid_gamma->modified )
		{
			vid_ref->modified = true;
			/* Daniel
			if ( stricmp( gl_driver->string, "3dfxgl" ) == 0 )
			{
				char envbuffer[1024];
				float g;

				vid_ref->modified = true;

				g = 2.00 * ( 0.8 - ( vid_gamma->value - 0.5 ) ) + 1.0F;
				Com_sprintf( envbuffer, sizeof(envbuffer), "SSTV2_GAMMA=%f", g );
				putenv( envbuffer );
				Com_sprintf( envbuffer, sizeof(envbuffer), "SST_GAMMA=%f", g );
				putenv( envbuffer );

				vid_gamma->modified = false;
			}*/
		}

		if ( gl_driver->modified )
			vid_ref->modified = true;
	}

	M_ForceMenuOff();
}

static void CancelChanges( void *unused )
{
	extern void M_PopMenu( void );

	M_PopMenu();
}

/*
** VID_MenuInit
*/
void VID_MenuInit( void )
{
	static const char *resolutions[] = 
	{
		"[320 240  ]",
		"[400 300  ]",
		"[512 384  ]",
		"[640 480  ]",
		"[800 600  ]",
		"[960 720  ]",
		"[1024 768 ]",
		"[1152 864 ]",
		"[1280 960 ]",
		"[1600 1200]",
		"[2048 1536]",
		0
	};
	static const char *refs[] =
	{
//		"[software      ]", //DANIEL
		"[default OpenGL]",
//		"[3Dfx OpenGL   ]",
//		"[PowerVR OpenGL]",
//		"[Rendition OpenGL]",
		0
	};
	static const char *yesno_names[] =
	{
		"no",
		"yes",
		0
	};

	//DANIEL
	static const char *particle_names[] =
	{
		"sprite",
		"hardware points",
		0
	};

	//gl_bitdepth
	static const char *bitdepth_names[] =
	{
		"desktop",
		"16 bit",
		"32 bit",
		0
	};

	static const char *texfilter_names[] =
	{
		"bilinear",
		"trilinear",
		0
	};
	static const char *texfilter_strings[] =
	{
		"GL_LINEAR_MIPMAP_NEAREST",
		"GL_LINEAR_MIPMAP_LINEAR",
		0
	};

	int DanielTexturemodeValue = 0;

	int i;
	int ydaniel = 0;
	if ( !gl_driver )
		gl_driver = Cvar_Get( "gl_driver", "opengl32", 0 );
	if ( !gl_picmip )
		gl_picmip = Cvar_Get( "gl_picmip", "0", 0 );
	if ( !gl_mode )
		gl_mode = Cvar_Get( "gl_mode", "3", 0 );
	if ( !sw_mode )
		sw_mode = Cvar_Get( "sw_mode", "0", 0 );
	if ( !gl_ext_palettedtexture )
		gl_ext_palettedtexture = Cvar_Get( "gl_ext_palettedtexture", "0", CVAR_ARCHIVE ); //Daniel  var 1

	if ( !gl_ext_pointparameters ) //DANIEL
		gl_ext_pointparameters = Cvar_Get( "gl_ext_pointparameters", "0", CVAR_ARCHIVE ); //Daniel  var 1

	if ( !gl_dynamic ) //DANIEL
		gl_dynamic = Cvar_Get( "gl_dynamic", "1", CVAR_ARCHIVE );

	if ( !gl_shadows ) //DANIEL
		gl_shadows = Cvar_Get( "gl_shadows", "0", CVAR_ARCHIVE );

	if ( !gl_bitdepth ) //DANIEL
		gl_bitdepth = Cvar_Get( "gl_bitdepth", "0", CVAR_ARCHIVE ); //Daniel  var 1

	if ( !gl_texturemode ) //DANIEL
		gl_texturemode = Cvar_Get( "gl_texturemode", "GL_LINEAR_MIPMAP_NEAREST", CVAR_ARCHIVE );

	// Kontrollera texturemode
	if ( strcmp( gl_texturemode->string, "GL_LINEAR_MIPMAP_NEAREST" ) == 0 )
	{
		DanielTexturemodeValue = 0;
	}
	else if ( strcmp( gl_texturemode->string, "GL_LINEAR_MIPMAP_LINEAR" ) == 0 )
	{
		DanielTexturemodeValue = 1;
	}
	else
	{
		DanielTexturemodeValue = 0;
	}


	if ( !gl_finish )
		gl_finish = Cvar_Get( "gl_finish", "0", CVAR_ARCHIVE );

	if ( !sw_stipplealpha )
		sw_stipplealpha = Cvar_Get( "sw_stipplealpha", "0", CVAR_ARCHIVE );

	s_mode_list[SOFTWARE_MENU].curvalue = sw_mode->value;
	s_mode_list[OPENGL_MENU].curvalue = gl_mode->value;

	if ( !scr_viewsize )
		scr_viewsize = Cvar_Get ("viewsize", "100", CVAR_ARCHIVE);

	s_screensize_slider[SOFTWARE_MENU].curvalue = scr_viewsize->value/10;
	s_screensize_slider[OPENGL_MENU].curvalue = scr_viewsize->value/10;

	//DANIEL se till att vi aldrig laddar ref_soft.dll
	/*if ( strcmp( vid_ref->string, "soft" ) == 0 )
	{
		s_current_menu_index = SOFTWARE_MENU;
		s_ref_list[0].curvalue = s_ref_list[1].curvalue = REF_SOFT;
	}
	else 
	*/
	if ( strcmp( vid_ref->string, "gl" ) == 0 )
	{
		s_current_menu_index = OPENGL_MENU;
		s_ref_list[s_current_menu_index].curvalue = REF_OPENGL;
	}

	s_opengl_menu.x = viddef.width * 0.50;
	s_opengl_menu.nitems = 0;

	s_mode_list[OPENGL_MENU].generic.type = MTYPE_SPINCONTROL;
	s_mode_list[OPENGL_MENU].generic.name = "video mode";
	s_mode_list[OPENGL_MENU].generic.x = 0;
	s_mode_list[OPENGL_MENU].generic.y = -20;
	s_mode_list[OPENGL_MENU].itemnames = resolutions;

	// DANIEL gl_ext_pointparameters
	s_bitdepth_type_box.generic.type = MTYPE_SPINCONTROL;
	s_bitdepth_type_box.generic.x	= 0;
	s_bitdepth_type_box.generic.y	= -10;
	s_bitdepth_type_box.generic.name	= "colour depth";
	s_bitdepth_type_box.curvalue = (gl_bitdepth->value)/16;
	s_bitdepth_type_box.itemnames = bitdepth_names;
	// DANIEL END

	s_screensize_slider[OPENGL_MENU].generic.type	= MTYPE_SLIDER;
	s_screensize_slider[OPENGL_MENU].generic.x		= 0;
	s_screensize_slider[OPENGL_MENU].generic.y		= 0;
	s_screensize_slider[OPENGL_MENU].generic.name	= "screen size";
	s_screensize_slider[OPENGL_MENU].minvalue = 3;
	s_screensize_slider[OPENGL_MENU].maxvalue = 12;
	s_screensize_slider[OPENGL_MENU].generic.callback = ScreenSizeCallback;

	s_brightness_slider[OPENGL_MENU].generic.type	= MTYPE_SLIDER;
	s_brightness_slider[OPENGL_MENU].generic.x	= 0;
	s_brightness_slider[OPENGL_MENU].generic.y	= 10;
	s_brightness_slider[OPENGL_MENU].generic.name	= "brightness";
	s_brightness_slider[OPENGL_MENU].generic.callback = BrightnessCallback;
	s_brightness_slider[OPENGL_MENU].minvalue = 5;
	s_brightness_slider[OPENGL_MENU].maxvalue = 13;
	s_brightness_slider[OPENGL_MENU].curvalue = ( 1.3 - vid_gamma->value + 0.5 ) * 10;

	s_tq_slider.generic.type	= MTYPE_SLIDER;
	s_tq_slider.generic.x		= 0;
	s_tq_slider.generic.y		= 20;
	s_tq_slider.generic.name	= "texture quality";
	s_tq_slider.minvalue = 0;
	s_tq_slider.maxvalue = 3;
	s_tq_slider.curvalue = 3-gl_picmip->value;

	s_fs_box[OPENGL_MENU].generic.type = MTYPE_SPINCONTROL;
	s_fs_box[OPENGL_MENU].generic.x	= 0;
	s_fs_box[OPENGL_MENU].generic.y	= 30;
	s_fs_box[OPENGL_MENU].generic.name	= "fullscreen";
	s_fs_box[OPENGL_MENU].itemnames = yesno_names;
	s_fs_box[OPENGL_MENU].curvalue = vid_fullscreen->value;

	s_finish_box.generic.type = MTYPE_SPINCONTROL;
	s_finish_box.generic.x	= 0;
	s_finish_box.generic.y	= 40;
	s_finish_box.generic.name	= "sync every frame";
	s_finish_box.curvalue = gl_finish->value;
	s_finish_box.itemnames = yesno_names;

	// DANIEL gl_ext_pointparameters
/*	s_particle_type_box.generic.type = MTYPE_SPINCONTROL;
	s_particle_type_box.generic.x	= 0;
	s_particle_type_box.generic.y	= 50;
	s_particle_type_box.generic.name	= "particles";
	s_particle_type_box.curvalue = gl_ext_pointparameters->value;
	s_particle_type_box.itemnames = particle_names;*/
	// DANIEL END

	// DANIEL s_dynamiclight_type_box
	s_dynamiclight_type_box.generic.type = MTYPE_SPINCONTROL;
	s_dynamiclight_type_box.generic.x	= 0;
	s_dynamiclight_type_box.generic.y	= 50;
	s_dynamiclight_type_box.generic.name	= "dynamic lights";
	s_dynamiclight_type_box.curvalue = gl_dynamic->value;
	s_dynamiclight_type_box.itemnames = yesno_names;

	s_texturemode_type_box.generic.type = MTYPE_SPINCONTROL;
	s_texturemode_type_box.generic.x	= 0;
	s_texturemode_type_box.generic.y	= 60;
	s_texturemode_type_box.generic.name	= "texture filter";
	s_texturemode_type_box.curvalue = DanielTexturemodeValue;
	s_texturemode_type_box.itemnames = texfilter_names;

	s_shadows_type_box.generic.type = MTYPE_SPINCONTROL;
	s_shadows_type_box.generic.x	= 0;
	s_shadows_type_box.generic.y	= 70;
	s_shadows_type_box.generic.name	= "shadows";
	s_shadows_type_box.curvalue = gl_shadows->value;
	s_shadows_type_box.itemnames = yesno_names;
	// DANIEL END


	s_defaults_action[OPENGL_MENU].generic.type = MTYPE_ACTION;
	s_defaults_action[OPENGL_MENU].generic.name = "reset to defaults";
	s_defaults_action[OPENGL_MENU].generic.x    = 0;
	s_defaults_action[OPENGL_MENU].generic.y    = 90;
	s_defaults_action[OPENGL_MENU].generic.callback = ResetDefaults;

	s_cancel_action[OPENGL_MENU].generic.type = MTYPE_ACTION;
	s_cancel_action[OPENGL_MENU].generic.name = "cancel";
	s_cancel_action[OPENGL_MENU].generic.x    = 0;
	s_cancel_action[OPENGL_MENU].generic.y    = 100;
	s_cancel_action[OPENGL_MENU].generic.callback = CancelChanges;

	Menu_AddItem( &s_opengl_menu, ( void * ) &s_mode_list[OPENGL_MENU] );
	Menu_AddItem( &s_opengl_menu, ( void * ) &s_bitdepth_type_box );
	Menu_AddItem( &s_opengl_menu, ( void * ) &s_screensize_slider[OPENGL_MENU] );
	Menu_AddItem( &s_opengl_menu, ( void * ) &s_brightness_slider[OPENGL_MENU] );
	Menu_AddItem( &s_opengl_menu, ( void * ) &s_tq_slider );
	Menu_AddItem( &s_opengl_menu, ( void * ) &s_fs_box[OPENGL_MENU] );
	Menu_AddItem( &s_opengl_menu, ( void * ) &s_finish_box );
//	Menu_AddItem( &s_opengl_menu, ( void * ) &s_particle_type_box );
	Menu_AddItem( &s_opengl_menu, ( void * ) &s_dynamiclight_type_box );
	Menu_AddItem( &s_opengl_menu, ( void * ) &s_texturemode_type_box );
	Menu_AddItem( &s_opengl_menu, ( void * ) &s_shadows_type_box );
	Menu_AddItem( &s_opengl_menu, ( void * ) &s_defaults_action[OPENGL_MENU] );
	Menu_AddItem( &s_opengl_menu, ( void * ) &s_cancel_action[OPENGL_MENU] );

	Menu_Center( &s_opengl_menu );
	s_opengl_menu.x -= 8;
}

/*
================
VID_MenuDraw
================
*/
void VID_MenuDraw (void)
{
	int w, h;

	if ( s_current_menu_index == 0 )
		s_current_menu = &s_software_menu;
	else
		s_current_menu = &s_opengl_menu;

	/*
	** draw the banner
	*/
	re.DrawGetPicSize( &w, &h, "m_banner_video" );
	re.DrawPic( viddef.width / 2 - w / 2, viddef.height /2 - 120, "m_banner_video" );

	/*
	** move cursor to a reasonable starting position
	*/
	Menu_AdjustCursor( s_current_menu, 1 );

	/*
	** draw the menu
	*/
	Menu_Draw( s_current_menu );
}

/*
================
VID_MenuKey
================
*/
const char *VID_MenuKey( int key )
{
	menuframework_s *m = s_current_menu;
	static const char *sound = "menu/menu1.wav";

	switch ( key )
	{
	case K_ESCAPE:
		ApplyChanges( 0 );
		return NULL;
	case K_KP_UPARROW:
	case K_UPARROW:
		m->cursor--;
		Menu_AdjustCursor( m, -1 );
		break;
	case K_KP_DOWNARROW:
	case K_DOWNARROW:
		m->cursor++;
		Menu_AdjustCursor( m, 1 );
		break;
	case K_KP_LEFTARROW:
	case K_LEFTARROW:
		Menu_SlideItem( m, -1 );
		break;
	case K_KP_RIGHTARROW:
	case K_RIGHTARROW:
		Menu_SlideItem( m, 1 );
		break;
	case K_KP_ENTER:
	case K_ENTER:
		if ( !Menu_SelectItem( m ) )
			ApplyChanges( NULL );
		break;
	}

	return sound;
}


